Package: mcmc
Version: 0.9-8
Date: 2023-11-14
Title: Markov Chain Monte Carlo
Author: Charles J. Geyer <geyer@umn.edu> and Leif T. Johnson
     <ltjohnson@google.com>
Maintainer: Charles J. Geyer <geyer@umn.edu>
Depends: R (>= 3.6.0)
Imports: stats
Suggests: xtable, Iso
ByteCompile: TRUE
Description: Simulates continuous distributions of random vectors using
    Markov chain Monte Carlo (MCMC).  Users specify the distribution by an
    R function that evaluates the log unnormalized density.  Algorithms
    are random walk Metropolis algorithm (function metrop), simulated
    tempering (function temper), and morphometric random walk Metropolis
    (Johnson and Geyer, 2012, <doi:10.1214/12-AOS1048>,
    function morph.metrop),
    which achieves geometric ergodicity by change of variable.
License: MIT + file LICENSE
URL: http://www.stat.umn.edu/geyer/mcmc/,
        https://github.com/cjgeyer/mcmc
NeedsCompilation: yes
Packaged: 2023-11-15 05:41:20 UTC; geyer
Repository: CRAN
Date/Publication: 2023-11-16 22:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:46:53 UTC; unix
Archs: mcmc.so.dSYM
