
useDynLib(mcmc, .registration = TRUE, .fixes = "C_")

export(metrop)
export(morph)
export(olbm)
export(initseq)
export(temper)
export(morph.metrop)
export(morph)
export(morph.identity)

S3method(metrop, metropolis)
S3method(metrop, "function")
S3method(morph.metrop, morph.metropolis)
S3method(morph.metrop, "function")
S3method(temper, tempering)
S3method(temper, "function")

importFrom(stats, runif)
